@@ -1,8 +1,6 @@
 #ifndef R2_CONS_H
 #define R2_CONS_H
 
-#define HAVE_DIETLINE 1
-
 #ifdef __cplusplus
 extern "C" {
 #endif
@@ -127,6 +125,7 @@ enum {
 	R_CONS_PAL_GUI_BACKGROUND,
 	R_CONS_PAL_GUI_ALT_BACKGROUND,
 	R_CONS_PAL_GUI_BORDER,
+	R_CONS_PAL_LINEHL,
 	R_CONS_PAL_GRAPH_BOX,
 	R_CONS_PAL_GRAPH_BOX2,
 	R_CONS_PAL_GRAPH_BOX3,
@@ -206,7 +205,8 @@ typedef struct r_cons_palette_t {
 	RColor gui_background;
 	RColor gui_alt_background;
 	RColor gui_border;
-	RColor highlight;
+	RColor wordhl;
+	RColor linehl;
 
 	/* Graph colors */
 	RColor graph_box;
@@ -270,7 +270,8 @@ typedef struct r_cons_printable_palette_t {
 	char *gui_background;
 	char *gui_alt_background;
 	char *gui_border;
-	char *highlight;
+	char *wordhl;
+	char *linehl;
 
 	/* graph colors */
 	char *graph_box;
@@ -304,8 +305,9 @@ typedef struct r_cons_canvas_t {
 	int h;
 	int x;
 	int y;
-	char *b;
-	int blen;
+	char **b;
+	int *blen;
+	int *bsize;
 	const char * attr;//The current attr (inserted on each write)
 	RConsCanvasAttr * attrs;// all the different attributes
 	int attrslen;
@@ -366,15 +368,25 @@ typedef struct r_cons_canvas_t {
 
 typedef char *(*RConsEditorCallback)(void *core, const char *file, const char *str);
 typedef int (*RConsClickCallback)(void *core, int x, int y);
+typedef void (*RConsBreakCallback)(void *core);
+typedef void *(*RConsSleepBeginCallback)(void *core);
+typedef void (*RConsSleepEndCallback)(void *core, void *user);
 
-typedef struct r_cons_t {
+typedef struct r_cons_context_t {
 	RConsGrep grep;
 	RStack *cons_stack;
-	RStack *break_stack;
 	char *buffer;
-	//int line;
 	int buffer_len;
 	int buffer_sz;
+
+	bool breaked;
+	RStack *break_stack;
+	RConsEvent event_interrupt;
+	void *event_interrupt_data;
+} RConsContext;
+
+typedef struct r_cons_t {
+	RConsContext *context;
 	char *lastline;
 	int is_html;
 	int is_interactive;
@@ -387,20 +399,22 @@ typedef struct r_cons_t {
 	int force_columns;
 	int fix_rows;
 	int fix_columns;
-	bool breaked;
 	bool break_lines;
 	int noflush;
 	FILE *fdin; // FILE? and then int ??
 	int fdout; // only used in pipe.c :?? remove?
 	const char *teefile;
 	int (*user_fgets)(char *buf, int len);
-	RConsEvent event_interrupt;
 	RConsEvent event_resize;
-	void *data;
 	void *event_data;
 	int mouse_event;
 
-	RConsEditorCallback editor;
+	RConsEditorCallback cb_editor;
+	RConsBreakCallback cb_break;
+	RConsSleepBeginCallback cb_sleep_begin;
+	RConsSleepEndCallback cb_sleep_end;
+	RConsClickCallback cb_click;
+
 	void *user; // Used by <RCore*>
 #if __UNIX__ || __CYGWIN__ && !defined(MINGW32)
 	struct termios term_raw, term_buf;
@@ -422,7 +436,6 @@ typedef struct r_cons_t {
 	struct r_line_t *line;
 	const char **vline;
 	int refcnt;
-	RConsClickCallback onclick;
 	bool newline;
 #if __WINDOWS__ && !__CYGWIN__
 	bool ansicon;
@@ -438,7 +451,7 @@ typedef struct r_cons_t {
 	bool grep_color;
 	bool use_tts;
 	bool filter;
-	char* (*rgbstr)(char *str, ut64 addr);
+	char* (*rgbstr)(char *str, size_t sz, ut64 addr);
 } RCons;
 
 // XXX THIS MUST BE A SINGLETON AND WRAPPED INTO RCons */
@@ -471,9 +484,9 @@ typedef struct r_cons_t {
 #define Color_INVERT       "\x1b[7m"
 #define Color_INVERT_RESET "\x1b[27m"
 /* plain colors */
-#define Color_RESET      "\x1b[0m"
-#define Color_NOBGRESET  "\x1b[22;24;25;27;28;39m"
-#define Color_BGRESET    "\x1b[49m"
+#define Color_RESET      "\x1b[0m" /* reset all */
+#define Color_RESET_NOBG "\x1b[22;24;25;27;28;39m"  /* Reset everything except background */
+#define Color_RESET_BG   "\x1b[49m"
 #define Color_BLACK      "\x1b[30m"
 #define Color_BGBLACK    "\x1b[40m"
 #define Color_RED        "\x1b[31m"
@@ -615,7 +628,7 @@ R_API void r_cons_canvas_line(RConsCanvas *c, int x, int y, int x2, int y2, RCan
 R_API void r_cons_canvas_line_diagonal(RConsCanvas *c, int x, int y, int x2, int y2, RCanvasLineStyle *style);
 R_API void r_cons_canvas_line_square(RConsCanvas *c, int x, int y, int x2, int y2, RCanvasLineStyle *style);
 R_API int r_cons_canvas_resize(RConsCanvas *c, int w, int h);
-R_API void r_cons_canvas_fill(RConsCanvas *c, int x, int y, int w, int h, char ch, int replace);
+R_API void r_cons_canvas_fill(RConsCanvas *c, int x, int y, int w, int h, char ch);
 R_API void r_cons_canvas_line_square_defined (RConsCanvas *c, int x, int y, int x2, int y2, RCanvasLineStyle *style, int bendpoint, int isvert);
 R_API void r_cons_canvas_line_back_edge (RConsCanvas *c, int x, int y, int x2, int y2, RCanvasLineStyle *style, int ybendpoint1, int xbendpoint, int ybendpoint2, int isvert);
 
@@ -623,12 +636,15 @@ R_API RCons *r_cons_new(void);
 R_API RCons *r_cons_singleton(void);
 R_API RCons *r_cons_free(void);
 R_API char *r_cons_lastline(int *size);
+R_API char *r_cons_lastline_utf8_ansi_len(int *len);
 
 typedef void (*RConsBreak)(void *);
 R_API void r_cons_break_end(void);
 R_API bool r_cons_is_breaked(void);
 R_API void r_cons_break_timeout(int timeout);
 R_API void r_cons_breakword(const char *s);
+R_API void *r_cons_sleep_begin(void);
+R_API void r_cons_sleep_end(void *user);
 
 /* pipe */
 R_API int r_cons_pipe_open(const char *file, int fdn, int append);
@@ -640,6 +656,11 @@ R_API int r_cons_w32_print(const ut8 *ptr, int len, int empty);
 
 R_API void r_cons_push(void);
 R_API void r_cons_pop(void);
+R_API RConsContext *r_cons_context_new(void);
+R_API void r_cons_context_free(RConsContext *context);
+R_API void r_cons_context_load(RConsContext *context);
+R_API void r_cons_context_reset();
+R_API bool r_cons_context_is_main();
 R_API void r_cons_break_pop(void);
 R_API void r_cons_break_push(RConsBreak cb, void*user);
 R_API void r_cons_break_clear(void);
@@ -692,6 +713,7 @@ R_API int r_cons_controlz(int ch);
 R_API int r_cons_readchar(void);
 R_API bool r_cons_readpush(const char *str, int len);
 R_API void r_cons_readflush();
+R_API void r_cons_switchbuf(bool active);
 R_API int r_cons_readchar_timeout(ut32 usec);
 R_API int r_cons_any_key(const char *msg);
 R_API int r_cons_eof(void);
@@ -706,6 +728,7 @@ R_API void r_cons_pal_random(void);
 R_API RColor r_cons_pal_get(const char *key);
 R_API RColor r_cons_pal_get_i(int index);
 R_API const char *r_cons_pal_get_name(int index);
+R_API int r_cons_pal_len();
 R_API int r_cons_rgb_parse(const char *p, ut8 *r, ut8 *g, ut8 *b, ut8 *a);
 R_API char *r_cons_rgb_tostring(ut8 r, ut8 g, ut8 b);
 R_API void r_cons_pal_list(int rad, const char *arg);
@@ -734,8 +757,8 @@ R_API int r_cons_grepbuf(char *buf, int len);
 R_API void r_cons_rgb(ut8 r, ut8 g, ut8 b, ut8 a);
 R_API void r_cons_rgb_fgbg(ut8 r, ut8 g, ut8 b, ut8 R, ut8 G, ut8 B);
 R_API void r_cons_rgb_init(void);
-R_API char *r_cons_rgb_str(char *outstr, RColor *rcolor);
-R_API char *r_cons_rgb_str_off(char *outstr, ut64 off);
+R_API char *r_cons_rgb_str(char *outstr, size_t sz, RColor *rcolor);
+R_API char *r_cons_rgb_str_off(char *outstr, size_t sz, ut64 off);
 R_API void r_cons_color(int fg, int r, int g, int b);
 R_API RColor r_cons_color_random(ut8 alpha);
 R_API void r_cons_invert(int set, int color);
@@ -755,6 +778,8 @@ R_API const char* r_cons_get_rune(const ut8 ch);
 #define R_LINE_BUFSIZE 4096
 #define R_LINE_HISTSIZE 256
 
+#define R_EDGES_X_INC 4
+
 typedef struct r_line_hist_t {
 	char **data;
 	int size;
@@ -780,12 +805,18 @@ typedef struct r_line_comp_t {
 } RLineCompletion;
 
 typedef char* (*RLineEditorCb)(void *core, const char *str);
+typedef int (*RLineHistoryUpCb)(RLine* line);
+typedef int (*RLineHistoryDownCb)(RLine* line);
 
 struct r_line_t {
 	RLineCompletion completion;
-	RLineHistory history;
 	RLineBuffer buffer;
-	RLineEditorCb editor_cb;
+	RLineHistory history;
+	/* callbacks */
+	RLineHistoryUpCb cb_history_up;
+	RLineHistoryDownCb cb_history_down;
+	RLineEditorCb cb_editor;
+	/* state , TODO: use more bool */
 	int echo;
 	int has_echo;
 	char *prompt;
@@ -796,6 +827,8 @@ struct r_line_t {
 	int (*hist_down)(void *user);
 	char *contents;
 	bool zerosep;
+	bool offset_prompt;
+	int offset_index;
 }; /* RLine */
 
 #ifdef R_API
@@ -820,6 +853,10 @@ R_API void r_line_label_show(void);
 R_API int r_line_hist_list(void);
 R_API const char *r_line_hist_get(int n);
 
+R_API int r_line_set_hist_callback(RLine *line, RLineHistoryUpCb cb_up, RLineHistoryDownCb cb_down);
+R_API int cmd_history_up(RLine *line);
+R_API int cmd_history_down(RLine *line);
+
 #define R_CONS_INVERT(x,y) (y? (x?Color_INVERT: Color_INVERT_RESET): (x?"[":"]"))
 
 #endif
@@ -869,8 +906,10 @@ typedef struct r_ascii_graph_t {
 	bool is_tiny;
 	bool is_dis;
 	int edgemode;
+	bool altedgepos;
 	int mode;
 	bool is_callgraph;
+	bool is_interactive;
 	int zoom;
 	int movspeed;
 
@@ -909,12 +948,27 @@ R_API void r_agraph_add_edge(const RAGraph *g, RANode *a, RANode *b);
 R_API void r_agraph_add_edge_at(const RAGraph *g, RANode *a, RANode *b, int nth);
 R_API void r_agraph_del_edge(const RAGraph *g, RANode *a, RANode *b);
 R_API void r_agraph_print(RAGraph *g);
+R_API void r_agraph_print_json(RAGraph *g);
 R_API Sdb *r_agraph_get_sdb(RAGraph *g);
 R_API void r_agraph_foreach(RAGraph *g, RANodeCallback cb, void *user);
 R_API void r_agraph_foreach_edge(RAGraph *g, RAEdgeCallback cb, void *user);
 R_API void r_agraph_set_curnode(RAGraph *g, RANode *node);
 #endif
 
+typedef struct r_panels_t {
+	RConsCanvas *can;
+	RPanel *panel;
+	int n_panels;
+	int columnWidth;
+	int layout;
+	int menu_pos;
+	int menu_x;
+	int menu_y;
+	int callgraph;
+	int curnode;
+	bool isResizing;
+} RPanels;
+
 #ifdef __cplusplus
 }
 #endif
